﻿$separatorLine = "--------------------------------------------------------------"
$indent = "    "

function GetQueryForInstalledApp
{
    param(
        [parameter(mandatory=$true, position=0)]
        [ValidateNotNullOrEmpty()]
        [bool] 
        $is64,
        [parameter(mandatory=$true, position=1)]
        [ValidateNotNullOrEmpty()]
        [string] 
        $prodID,
        [parameter(mandatory=$true, position=2)]
        [ValidateNotNullOrEmpty()]
        [string] 
        $appVersionCriteria,
        [parameter(mandatory=$false, position=3)]
        [string] 
        $appNameCriteria
    )

    #Write-Host "App criteria=$appNameCriteria"
    #Write-Host "App criteria=$appVersionCriteria"
    if ($is64)
    {
        $addRemoveProgramTable = "SMS_G_System_ADD_REMOVE_PROGRAMS_64"
    }
    else
    {
        $addRemoveProgramTable = "SMS_G_System_ADD_REMOVE_PROGRAMS"
    }

    $condition1 = "$addRemoveProgramTable.ProdID like '$prodID'"
    $condition2 = "$addRemoveProgramTable.Version >= '$appVersionCriteria'"
    $condition3 = ""
    if (!$appNameCriteria)
    {
        if (!($appNameCriteria -eq ""))
        {
            $condition3 = SMS_G_System_ADD_REMOVE_PROGRAMS.DisplayName like '$appNameCriteria'
        }
    }

    
    $queryString = 
    "select SMS_R_SYSTEM.ResourceID,
SMS_R_SYSTEM.ResourceType,
SMS_R_SYSTEM.Name,`
SMS_R_SYSTEM.SMSUniqueIdentifier,
SMS_R_SYSTEM.ResourceDomainORWorkgroup,
SMS_R_SYSTEM.Client
from SMS_R_System
    inner join SMS_G_System_COMPUTER_SYSTEM on SMS_G_System_COMPUTER_SYSTEM.ResourceId = SMS_R_System.ResourceId  
    inner join $addRemoveProgramTable on $addRemoveProgramTable.ResourceID = SMS_G_System_COMPUTER_SYSTEM.ResourceID
where $condition1 and $condition2
"

    if (!($condition3 -eq ""))
    {
        $queryString = $queryString + " and $condition3"
    }
    return $queryString
}

function Move-ObjectToFolder
{
    param(
        [parameter(mandatory=$true, position=0)]
        [ValidateNotNullOrEmpty()]
        [object] 
        $object,
        [parameter(mandatory=$true, position=1)]
        [ValidateNotNullOrEmpty()]
        [string] 
        $folderPath
    )
    Write-Host "Moving $($object.Name) to $folderPath"
    Move-CMObject -InputObject $object -FolderPath $folderPath
    Write-Host "=>Object $($object.Name) has been moved to $folderPath."

}

function Create-FolderInDeviceCollection
{
    param (
        [parameter(mandatory=$true, position=0)]
        [ValidateNotNullOrEmpty()]
        [string] 
        $folderName,
        [parameter(mandatory=$true, position=1)]
        [ValidateNotNullOrEmpty()]
        [string] 
        $path
    )

    Write-Host $separatorLine
    Write-Host "Create-Folder"
    Write-Host "$indent FolderName=$folderName"
    Write-Host "$indent Path=$path"
   
    # Check if HP folder exists.
    $searchString = "Name='$folderName'"

    $folder = Get-Item -Path "$($path)\$($folderName)" -ErrorAction "SilentlyContinue"

#    Query "Select * from SMS_ObjectContainerNode where Name='$($folderName)'"
     
    if ($folder)
    {
        Write-Host "=>Folder already exists. Will not create."
    }
    else
    {
        # Create HP folder.
        Write-Host "Creating folder $folderName"
        New-item -Name $folderName -Path $path 
        Write-Host "=>Folder $folderName has been created"
    }
}

function Create-Collection
{
    param (
        [parameter(mandatory=$true, position=0)]
        [ValidateNotNullOrEmpty()]
        [string] 
        $collectionName,
        [parameter(mandatory=$true, position=1)]
        [ValidateNotNullOrEmpty()]
        [string] 
        $limitingCollectionName,
        [parameter(mandatory=$true, position=2)]
        [ValidateNotNullOrEmpty()]
        [string] 
        $ruleQueryString,
        [parameter(mandatory=$true, position=3)]
        [ValidateNotNullOrEmpty()]
        [string] 
        $ruleName,
        [parameter(mandatory=$true, position=4)]
        [ValidateNotNullOrEmpty()]
        [string] 
        $folderPath,
        [parameter(mandatory=$true, position=5)]
        [ValidateNotNullOrEmpty()]
        [object] 
        $refreshSchedule
    )


    Write-Host $separatorLine
    Write-Host "Create-Collection" 
    Write-Host "$indent CollectionName=$collectionName"
    Write-Host "$indent LimitingCollection=$limitingCollectionName"
    Write-Host "$indent FolderPath=$folderPath"

    # Validate the collection name.
    if ([string]::IsNullOrEmpty($collectionName) -or [string]::IsNullOrEmpty($limitingCollectionName) -or [string]::IsNullOrEmpty($ruleQueryString) -or [string]::IsNullOrEmpty($folderPath)) {   
        Return
    }

    # Check if the collection already exists.
    $Collection = Get-CMDeviceCollection -Name $collectionName
    if ($collection) {
        Write-Host "=>Collection already exists. Will not create."
    }
    else
    {
        # Create the collection.
        Write-Host "Creating collection $collectionName"
        New-CMDeviceCollection -Name $collectionName -LimitingCollectionName $limitingCollectionName -RefreshSchedule $refreshSchedule

        $collection = Get-CMDeviceCollection -Name $collectionName

        # Move the collection to the desired folder.
        if ($collection)
        {
            Move-ObjectToFolder $collection $folderPath
        }

        # Add query rule to the collection
        Add-CMDeviceCollectionQueryMembershipRule -CollectionName $collectionName -QueryExpression $ruleQueryString -RuleName $ruleName
 

        # Update collection
        Invoke-CMDeviceCollectionUpdate -Name $collectionName

        Write-Host "=>Collection $collectionName has been created in $folderPath."
    }

}


function Create-SWPackage([string] $name, [string] $description, [string] $manufacturer, [string] $path, [string] $version, [string] $cmd, [string] $programName
, [string] $programRunType, [string] $runMode, [string] $runType, [bool] $userInteraction, [string] $workingDir, [String] $folderPath, [bool] $createPackageOnly = $false)
{

    Write-Host $separatorLine

    # Check if the package exists.
    $package = Get-CMPackage -Name $name
    if ($package) {
        Write-Host "=>Package $name already exists. Will not create."
        $package = Get-CMPackage -Name $name
    }
    else {
        # Not exist -> Create package
        New-CMPackage -Name $name -Description $description -Language $language -Manufacturer $manufacturer -Path $path -Version $version 
        $package = Get-CMPackage -Name $name
        if ($package) {
            Write-Host "=>Package $name has been created successfully."

            Move-ObjectToFolder $package $folderPath
        }
    }

    if ($package -and (!$createPackageOnly)) {
        # Check if program exists
        $program = Get-CMProgram -PackageId $package.PackageID -ProgramName $programName

        if ($program) {
            Write-Host "=>Program $programName already exists. Will not create."
        }
        else {
            # Create the program.
            New-CMProgram -CommandLine $cmd -PackageName $name -StandardProgramName $programName -ProgramRunType $programRunType -RunMode $runMode -RunType $runType -UserInteraction $userInteraction -WorkingDirectory $workingDir
            Write-Host "=>Program $programName has been created."
        }
    }
}

