#=====================================================================
# Filename:    CreateHPCollections.ps1
# Description: Create sample HP collections in Microsoft System Center
#              Configuration Manager. They can be helpful 
#              when working with HP systems.
#
# DISCLAIMER:
# TODO: Need to add copyright and disclaimer!
#
#=====================================================================

#Top of script
param ([String] $IACollections)

."$PSScriptRoot\CMHelper.ps1"

# Folder name and WMI query strings used in this script
$hpFolderName = "HP"

# All HP systems
$queryAllHP = "select SMS_R_SYSTEM.ResourceID,`
SMS_R_SYSTEM.ResourceType,`
SMS_R_SYSTEM.Name,`
SMS_R_SYSTEM.SMSUniqueIdentifier,`
SMS_R_SYSTEM.ResourceDomainORWorkgroup,`
SMS_R_SYSTEM.Client`
from SMS_R_System`   
inner join SMS_G_System_COMPUTER_SYSTEM on SMS_G_System_COMPUTER_SYSTEM.ResourceId = SMS_R_System.ResourceId  `
where `
(SMS_G_System_COMPUTER_SYSTEM.Manufacturer like 'Hewlett-Packard%'`
and SMS_G_System_COMPUTER_SYSTEM.Model not  like '%Proliant%') `
or SMS_G_System_COMPUTER_SYSTEM.Manufacturer like 'HP%'"

# Windows 10 Enterprise
$queryWindows10Ent = "select SMS_R_SYSTEM.ResourceID,`
    SMS_R_SYSTEM.ResourceType,`
    SMS_R_SYSTEM.Name,`
    SMS_R_SYSTEM.SMSUniqueIdentifier,`
    SMS_R_SYSTEM.ResourceDomainORWorkgroup,`
    SMS_R_SYSTEM.Client`
from SMS_R_System inner join SMS_G_System_Operating_system on SMS_R_System.ResourceID = SMS_G_System_Operating_system.ResourceID`
where Caption like '%Windows%10%Enterprise%'"

# HP systems with virtualization enabled

# All HP systems with TPM 1.2
$queryTPM1_2 = "select SMS_R_SYSTEM.ResourceID,`
    SMS_R_SYSTEM.ResourceType,`
    SMS_R_SYSTEM.Name,`
    SMS_R_SYSTEM.SMSUniqueIdentifier,`
    SMS_R_SYSTEM.ResourceDomainORWorkgroup,`
    SMS_R_SYSTEM.Client`
from SMS_R_System inner join SMS_G_System_COMPUTER_SYSTEM on SMS_G_System_COMPUTER_SYSTEM.ResourceId = SMS_R_System.ResourceId and SMS_G_System_COMPUTER_SYSTEM.Manufacturer like 'HP%'`
inner join SMS_G_System_TPM on SMS_R_System.ResourceId = SMS_G_System_TPM.ResourceId and SMS_G_System_TPM.SpecVersion like '1.2%'"

# All HP systems with TPM 2.0
$queryTPM2_0 = "select SMS_R_SYSTEM.ResourceID,`
    SMS_R_SYSTEM.ResourceType,`
    SMS_R_SYSTEM.Name,`
    SMS_R_SYSTEM.SMSUniqueIdentifier,`
    SMS_R_SYSTEM.ResourceDomainORWorkgroup,`
    SMS_R_SYSTEM.Client`
from SMS_R_System inner join SMS_G_System_COMPUTER_SYSTEM on SMS_G_System_COMPUTER_SYSTEM.ResourceId = SMS_R_System.ResourceId and SMS_G_System_COMPUTER_SYSTEM.Manufacturer like 'HP%'`
inner join SMS_G_System_TPM on SMS_R_System.ResourceId = SMS_G_System_TPM.ResourceId and SMS_G_System_TPM.SpecVersion like '2.0%'"

# All HP systems with Intel Authenticate Engine installed
$queryIAReady = "select SMS_R_SYSTEM.ResourceID,`
    SMS_R_SYSTEM.ResourceType,`
    SMS_R_SYSTEM.Name,`
    SMS_R_SYSTEM.SMSUniqueIdentifier,`
    SMS_R_SYSTEM.ResourceDomainORWorkgroup,`
    SMS_R_SYSTEM.Client`
from SMS_R_System inner join SMS_G_System_COMPUTER_SYSTEM on SMS_G_System_COMPUTER_SYSTEM.ResourceId = SMS_R_System.ResourceId and SMS_G_System_COMPUTER_SYSTEM.Manufacturer like 'HP%'`
inner join SMS_G_System_CM_IntelAuthenticatePolicies on SMS_R_System.ResourceId = SMS_G_System_CM_IntelAuthenticatePolicies.ResourceId`
 and SMS_G_System_CM_IntelAuthenticatePolicies.State = 'Active' "

# All HP systems with a valid Client Security - Intel Authenticate policy 
$queryIAValidPolicyInstalled = "select SMS_R_SYSTEM.ResourceID,`
    SMS_R_SYSTEM.ResourceType,`
    SMS_R_SYSTEM.Name,`
    SMS_R_SYSTEM.SMSUniqueIdentifier,`
    SMS_R_SYSTEM.ResourceDomainORWorkgroup,`
    SMS_R_SYSTEM.Client`
from SMS_R_System inner join SMS_G_System_COMPUTER_SYSTEM on SMS_G_System_COMPUTER_SYSTEM.ResourceId = SMS_R_System.ResourceId and SMS_G_System_COMPUTER_SYSTEM.Manufacturer like 'HP%'`
inner join SMS_G_System_CM_IntelAuthenticatePolicies on SMS_R_System.ResourceId = SMS_G_System_CM_IntelAuthenticatePolicies.ResourceId`
 and SMS_G_System_CM_IntelAuthenticatePolicies.IsValidPolicyInstalled = 'True' "

# client app versions to check for
$MIKClientVersion = "1.0.26.1"
$IAEngineVersion = "2.0.5.248"


#-------------------------------------------------------------
#                   CONNECT TO CONFIGMGR
#-------------------------------------------------------------
if ($EXIT_CONFIGMGR_NOT_DEFINED) { try {Remove-Variable -Name EXIT_CONFIGMGR_NOT_DEFINED -Force -ErrorAction SilentlyContinue } catch { } }
Set-Variable EXIT_CONFIGMGR_NOT_DEFINED -option ReadOnly -value 1
if ($EXIT_FAILED_TO_GET_SITE_CODE) { try {Remove-Variable -Name EXIT_FAILED_TO_GET_SITE_CODE -Force -ErrorAction SilentlyContinue } catch { } }
Set-Variable EXIT_FAILED_TO_GET_SITE_CODE -option ReadOnly -value 2

# Check if SMS admin UI path environment variable is defined.
if (!$env:SMS_ADMIN_UI_PATH)
{
    Write-Host "ConfigMgr environment is not defined."
    Exit($EXIT_CONFIGMGR_NOT_DEFINED)
}

# Import the ConfigMgr PowerShell module.
Import-Module $env:SMS_ADMIN_UI_PATH.Replace("\bin\i386","\bin\configurationmanager.psd1")

Write-Host "Retrieving site code"
$SiteCode = Get-PSDrive -PSProvider CMSITE

# Check if the site code is populated.
if (!$SiteCode)
{
    Write-Host "Error: Site code is null."
    Exit($EXIT_FAILED_TO_GET_SITE_CODE)
}

Write-Host "SiteCode=$SiteCode"

Set-Location "$($SiteCode.Name):\"
$SiteCodeName = $SiteCode.Name 

$path = "$($SiteCodeName):\DeviceCollection"

#-------------------------------------------------------------
#                CREATE FOLDER AND COLLECTIONS
#-------------------------------------------------------------
#Create HP folder under Device Collections
Create-FolderInDeviceCollection $hpFolderName $path

#------------------------------------------------------------------------------
# Create collection: All HP Systems
    $ruleName = "All HP Clients"
    $allHPCollectionName = "All HP Systems"
    $allClientCollectionName = "All Desktop and Server Clients" 
    $folderPath = "$path\$hpFolderName"

    $date = Get-Date -Format s
    $refreshSchedule = New-CMSchedule -Start ($date) -RecurInterval Days -RecurCount 1

Create-Collection $allHPCollectionName $allClientCollectionName $queryAllHP $ruleName $folderPath $refreshSchedule

$Collection = Get-CMDeviceCollection -Name $allHPCollectionName 


#------------------------------------------------------------------------------
# Create collection: All HP Systems with a particular MIK client version 

    $ruleName = "All HP systems with MIK Client version $MIKClientVersion"
    $allHPWithMIKClientCollectionName = "All HP Systems - MIK Client version $MIKClientVersion"
    $appProdID = "%6250351D-F14D-4A52-982A-1F0B4DEE29A3%"
    $appNameCriteria = ""

    $queryHPwithMIKClient = GetQueryForInstalledApp $false $appProdID $MIKClientVersion $appNameCriteria 
    #Write-Host "Query for MIK client=$queryHPwithMIKClient"

if (!($queryHPwithMIKClient -eq ""))
{
    $refreshSchedule = New-CMSchedule -Start ($date) -RecurInterval Days -RecurCount 1
    #Write-Host "$date $refreshSchedule"

    Create-Collection $allHPWithMIKClientCollectionName $allClientCollectionName $queryHPwithMIKClient $ruleName $folderPath $refreshSchedule

    $Collection = Get-CMDeviceCollection -Name $allHPWithMIKClientCollectionName 
}


#------------------------------------------------------------------------------
# Create collection: All HP Systems with TPM 1.2
    $ruleName = "All HP Clients - TPM 1.2"
    $allHPTPM1_2CollectionName = "All HP Systems - TPM 1.2"
    $allClientCollectionName = "All Desktop and Server Clients" #"All Systems"
    $folderPath = "$path\$hpFolderName"

    $date = Get-Date -Format s
    $refreshSchedule = New-CMSchedule -Start ($date) -RecurInterval Days -RecurCount 1

Create-Collection $allHPTPM1_2CollectionName $allClientCollectionName $queryTPM1_2 $ruleName $folderPath $refreshSchedule

$Collection = Get-CMDeviceCollection -Name $allHPCollectionName 

#------------------------------------------------------------------------------
# Create collection: All HP Systems with TPM 2.0
    $ruleName = "All HP Clients - TPM 2.0"
    $allHPTPM1_2CollectionName = "All HP Systems - TPM 2.0"
    $allClientCollectionName = "All Desktop and Server Clients" #"All Systems"
    $folderPath = "$path\$hpFolderName"

    $date = Get-Date -Format s
    $refreshSchedule = New-CMSchedule -Start ($date) -RecurInterval Days -RecurCount 1

Create-Collection $allHPTPM1_2CollectionName $allClientCollectionName $queryTPM2_0 $ruleName $folderPath $refreshSchedule

$Collection = Get-CMDeviceCollection -Name $allHPCollectionName 


#------------------------------------------------------------------------------
# Create collection: All HP Systems with a particular Intel Authenticate engine version
    $ruleName = "All HP systems with Intel Authenticate Engine version $IAEngineVersion or later"
    $allIAEngineInstalledCollectionName = "All HP Systems - Intel Authenticate Engine version $IAEngineVersion or later"
    $appProdID = "%31423228-4184-4B7E-914A-093C5A5E0D41%"
    $appNameCriteria = ""

    $queryIAEngineInstalled = GetQueryForInstalledApp $true $appProdID $IAEngineVersion $appNameCriteria 
    #Write-Host "Query for IA Engine installed=$queryIAEngineInstalled"

if (!($queryIAEngineInstalled -eq ""))
{
    $refreshSchedule = New-CMSchedule -Start ($date) -RecurInterval Days -RecurCount 1
    #Write-Host "$date $refreshSchedule"

    Create-Collection $allIAEngineInstalledCollectionName $allClientCollectionName $queryIAEngineInstalled $ruleName $folderPath $refreshSchedule

    $Collection = Get-CMDeviceCollection -Name $allIAEngineInstalledCollectionName 
}


Write-Host "IACollections=$IACollections"

if ($IACollections)
{
#------------------------------------------------------------------------------
# Create collection: Intel Authenticate  Ready
    $ruleName = "All HP Clients that has Intel Authenticate Engine installed"
    $allHPIAReadyCollectionName = "All HP Systems - Intel Authenticate  Ready"

    $refreshSchedule = New-CMSchedule -Start ($date) -RecurInterval Days -RecurCount 1

Create-Collection $allHPIAReadyCollectionName $allClientCollectionName $queryIAReady $ruleName $folderPath $refreshSchedule

$Collection = Get-CMDeviceCollection -Name $allHPIAReadyCollectionName 


#------------------------------------------------------------------------------
# Create collection: Intel Authenticate  Valid policy installed
    $ruleName = "All HP Clients that has HP Client Security with Intel Authenticate Policy installed"
    $allHPIAValidPolicyCollectionName = "All HP Systems - Intel Authenticate  Managed"

    $refreshSchedule = New-CMSchedule -Start ($date) -RecurInterval Days -RecurCount 1

Create-Collection $allHPIAValidPolicyCollectionName $allClientCollectionName $queryIAValidPolicyInstalled $ruleName $folderPath $refreshSchedule

$Collection = Get-CMDeviceCollection -Name $allHPIAValidPolicyCollectionName 

} #if IACollections


Remove-Variable -Name EXIT_CONFIGMGR_NOT_DEFINED -Force
Remove-Variable -Name EXIT_FAILED_TO_GET_SITE_CODE -Force